/* Name: PID-Controller with floats
 * Author: Michael
 * Copyright: 2018
 * License: <insert your license reference here>
 */

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "pid.h"
#include "uart.h"

char valueToPrint[6];
volatile uint16_t adcValue;
volatile uint16_t i=0;
volatile float voltage;
volatile uint16_t voltageForUart[500];
volatile uint8_t pwmForUart[sizeof(voltageForUart)/sizeof(uint16_t)];
volatile pid_controller_t myController;

#define SETVALUE 2.0
#define KP 800.0
#define KI 2.0
#define KD 0.4

int main(void)
{ 
	//UART einschalten
    uartSetup(38400UL);
    sendString_p(PSTR("UART init done!\r\n")); 
    //ADC auf interne Ref stellen
    ADMUX = (1 << REFS0)|(1 << REFS1);
    //ADC einschalten, PS auf 64
    ADCSRA = (1 << ADEN)|(1 << ADPS2)|(1 << ADPS1);
	// Auf Channel 5 einstellen
	ADMUX = (ADMUX & ~(0x1F)) | (5 & 0x1F);
    //Dummyread
    ADCSRA |= (1 << ADSC);
    while (ADCSRA & (1<<ADSC) ) {
    }
    // Interrupt und Autotrigger an
    ADCSRA |= (1 << ADIE)|(1 << ADATE);
    // Autotrigger auf Timer 1 Compare Match B
    ADCSRB = (1 << ADTS2)|(1 << ADTS0);
    sendString_p(PSTR("ADC init done!\r\n"));
    // PWM fuer OCR0B einstellen
    DDRD |= (1 << PD5);
    TCCR0A |= (1 << COM0B1)|(1 << WGM01)| (1 << WGM00);
    TCCR0B |= (1 << CS00);
    sendString_p(PSTR("PWM init done!\r\n"));
    // 5s warten damit der Kondensator vollständig entladen ist
    sendString_p(PSTR("Discharge capacitor!\r\n"));
    _delay_ms(5000);
    sendString_p(PSTR("Capacitor discharged!\r\n"));
    // Timer 1 einstellen für Compare Match, PID wird alle  1ms Aufgerufen
    ICR1 = 8000;
    TCCR1B |= (1 << WGM13)|(1 << WGM12)|(1 << CS10);
    TIMSK1 |= (1 << OCIE1B);
    sendString_p(PSTR("PID measure interval set to 1ms!\r\n"));
    // PID-Kontroller einstellen
    pid_init(KP, KI, KD, &myController);
    sendString_p(PSTR("PID-Controller init done!\r\n"));
    // Globale Interrupts einschalten
    sei();
    sendString("Init done!\r\n");
    sendString("========================================\r\n");
    for(;;){
    	// Die ersten 1000 Werte werden via UART übertragen
        if(i == sizeof(voltageForUart)/sizeof(uint16_t)){
        	sendString_p(PSTR("Cyclecount,PWM-Value, Voltage\r\n"));
            for(uint16_t j=0;j < sizeof(voltageForUart)/sizeof(uint16_t); j++){
            	itoa(j+1, valueToPrint,10);
            	sendString(valueToPrint);
            	sendString_p(PSTR(", "));
            	
            	itoa(pwmForUart[j], valueToPrint,10);
            	sendString(valueToPrint);
            	sendString_p(PSTR(", "));
            	
            	dtostrf(voltageForUart[j]/1000.0,
                    4,
                    2,
                    valueToPrint);
            
            	sendString(valueToPrint);
            	sendString_p(PSTR("\r\n"));
            }
            i++;
        }
        
    }
    return 0;   /* never reached */
}
ISR(TIMER1_COMPB_vect){
	OCR0B = pid_compute(SETVALUE, voltage, &myController);
	if(i < sizeof(voltageForUart)/sizeof(uint16_t)){
        voltageForUart[i]=voltage*1000;
        pwmForUart[i]=OCR0B;
        i++;
    }      
}
ISR(ADC_vect){
	voltage = (1.126/1024.0) * ADC * 11.0;
}