#include "pid.h"

void pid_init(float kp, float ki, float kd, pid_controller_t* controller){
    controller->kpFactor = kp;
    controller->kiFactor = ki;
    controller->kdFactor = kd;
    controller->lastValue = 0;
    controller->error = 0;
    controller->lastError = 0;
    controller->preLastError = 0;
}

uint8_t pid_compute(float sollWert, float istWert, pid_controller_t* controller){
    controller->error = sollWert - istWert;
    
    float y = controller->lastValue
    + (controller->kpFactor+controller->kiFactor+controller->kdFactor) * controller->error
    + (-controller->kpFactor-2*controller->kdFactor) * controller->lastError
    + controller->kdFactor * controller->preLastError;
    
    controller->preLastError = controller->lastError;
    controller->lastError = controller->error;
    controller->lastValue = y;
    
    if(y < 0) return 0;
    if(y > 0xff) return 0xff;
    
    return (uint8_t)y;
}